/**
 * Title: Expert 02 - Experimenting hierarchy issues
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

public class Expert02 extends Expert01 {

	// Remember that constructors are not hieritated, and here we have only 1 constructor.
	// For this reason, calling "new Expert02()" it is not possible  !!!
	public Expert02(int[] dimensions) {

		// Calling the parent constructor, which in turn calls "new Expert01()", even if
		// the constructor "new Expert02()" DOES NOT EXIST.
		super(dimensions);
	}

	// MAIN METHOD
    public static void main(String arg[]) throws Exception {

		int[] input = new int[2] ;
		if (arg.length > 0) input[0] = Integer.parseInt(arg[0]) ;
		if (arg.length > 1) input[1] = Integer.parseInt(arg[1]) ;

		Expert02 application = new Expert02(input) ;
		application.printState() ;

	}

	// This method OVERRIDES the method printState() defined in the parent class, hence
	// when it executes super.printState() the output will be different (shadowing)
	// Things are different for super.dimensions : this field is not overridden, hence
	// any reference to dimensions[] or super.dimensions[] gives the same result !!!
	// (infact Java creates an "hidden" instance of the superclass by default)
	public void printState() {

		System.out.print("**** EXECUTING THE CHILD METHOD ****\n") ;
		System.out.print("State of child (" + this + "): ") ;
		System.out.print("(X:" + dimensions[0] + ", ") ;
		System.out.print("Y:" + dimensions[1] + ")\n") ;
		System.out.print("State of parent (" + super.getClass().getName() + "): ") ;
		System.out.print("(X:" + super.dimensions[0] + ", ") ;
		System.out.print("Y:" + super.dimensions[1] + ")\n\n") ;

		System.out.print("**** CALLING THE PARENT METHOD ****\n") ;
		super.printState() ;
	}
}
