/**
 * Title: Expert 04 - Main class for testing access types into same package
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

public class Expert04 {

	// CLASS ATTRIBUTES
	public Expert05 parent ;
	public Expert06 child  ;

	public boolean[][] access = new boolean[2][4] ;		// can the object access the method/field?
	public boolean[][] has    = new boolean[2][4] ;		// has the object inherited the method/field?

	public String[] headings = {"Child (same package)", "Not Child (same package)"} ;	// mind identation
	public String[] labels 	= {"public   ", "protected", "package  ", "private  "} ;	// mind identation

	// Main method
	public static void main(String arg[]) throws Exception {

		Expert04 application = new Expert04() ;
		application.setState() ;

		if (arg.length > 0) {
			if (arg[0].equals("f")) application.fieldsTable() ;
			if (arg[0].equals("m")) application.methodsTable() ;
		}
	}

	// Creates the needed objects (parent and child)
	public void setState() {

		parent 	= new Expert05() ;
		child  	= new Expert06() ;
	}

	// Print the result of all possible access combinations (to methods)
	public void methodsTable() {

		// Testing access and inheritage for a "child class" (Expert06)
		access[0][0] = child.accessPublicMethod() ;
		access[0][1] = child.accessProtectedMethod() ;
		access[0][2] = child.accessPackageMethod() ;
		access[0][3] = child.accessPrivateMethod() ;

		has[0][0] = child.hasPublicMethod() ;
		has[0][1] = child.hasProtectedMethod() ;
		has[0][2] = child.hasPackageMethod() ;
		has[0][3] = child.hasPrivateMethod() ;

		// Testing access and inheritage for a "not child class" (this class)
		access[1] = testParentMethods() ;

		has[1][0] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][1] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][2] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][3] = false ;		// obvious (Expert04 does not extend Expert05)

		printResults("ACCESS METHOD ?",access) ;
		printResults("HAS METHOD ?",has) ;
	}

	// Print the result of all possible access combinations (to fields)
	public void fieldsTable() {

		// Testing access and inheritage for a "child class" (Expert06)
		access[0][0] = child.accessPublicField() ;
		access[0][1] = child.accessProtectedField() ;
		access[0][2] = child.accessPackageField() ;
		access[0][3] = child.accessPrivateField() ;

		has[0][0] = child.hasPublicField() ;
		has[0][1] = child.hasProtectedField() ;
		has[0][2] = child.hasPackageField() ;
		has[0][3] = child.hasPrivateField() ;

		// Testing access and inheritage for a "not child class" (this class)
		access[1] = testParentFields() ;

		has[1][0] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][1] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][2] = false ;		// obvious (Expert04 does not extend Expert05)
		has[1][3] = false ;		// obvious (Expert04 does not extend Expert05)

		printResults("ACCESS FIELD ?",access) ;
		printResults("HAS FIELD ?",has) ;
	}

	// Evaluate how a "not child" (this class) can access methods in the Expert05 object
	public boolean[] testParentMethods() {

		parent.printState() ;				// access public method ?
		int[] foo = parent.getState() ;		// access protected method ?
		parent.setState(1,2,3,4) ;			// access package method ?
		//parent.bundle() ;					// access private method ? (does not compile)

		boolean[] result = {true, true, true, false} ;
		return result ;
	}

	// Evaluate how a "not child" (this class) can access methods in the Expert05 object
	public boolean[] testParentFields() {

		int foo1 = parent.publicInt ;		// access public field ?
		int foo2 = parent.protectedInt ;	// access protected field ?
		int foo3 = parent.packageInt ;		// access package field ?
		//int foo4 = parent.privateInt ;	// access private field ? (does not compile)

		boolean[] result = {true, true, true, false} ;
		return result ;
	}

	// Print the general results on the screen
	public void printResults(String caption, boolean[][] results) {

		System.out.println("\n************* " + caption + " ********************\n") ;
		System.out.println("	    " + headings[0] + "   " + headings[1]) ;
		System.out.println() ;
		String blank = "                   " ;

		for (int i = 0 ; i < 4; i++) {

			System.out.print(labels[i] + "   ") ;
			System.out.print(results[0][i] + blank) ;
			System.out.print(results[1][i]) ;
			System.out.println() ;
		}

		System.out.println() ;
	}

} // End of class