/**
 * Title: Expert 09 - Practising with Input, Output, Streams, Writer and Readers.
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.io.*;

public class Expert09 {

	String fs = File.separator ;
	String folder = fs + "work" + fs + "java" + fs + "build" + fs ;
	String source = "C:" + folder + "input.txt" ;
	String sink1  = "C:" + folder + "output1.txt" ;
	String sink2  = "C:" + folder + "output2.txt" ;
	String sink3  = "C:" + folder + "output3.txt" ;

	// Main method
	public static void main(String args[]) throws Exception {

		Expert09 application = new Expert09();
	}

	// Basic constructor: it also perform all operations !
    public Expert09() throws IOException {

		System.out.println("\nCreating output files ...") ;
		System.out.println("Written dimensions: " + writeFiles() ) ;
		System.out.println() ;

		System.out.println("Reading portable data file ...") ;
		System.out.println("Read dimensions: " + readData() );
		System.out.println() ;
	}

	// Reading the 'source' data and creating sink1, sink2 and sink3.
	public int writeFiles() throws IOException {

		// These INPUT streams will wrapp those created in the try-catch block
		BufferedInputStream  input1 = null ;
		BufferedReader 		 input2 = null ;
		// These OUTPUT streams will wrapp those created in the try-catch block
        BufferedOutputStream output1 = null ;
		BufferedWriter 		 output2 = null ;
        DataOutputStream 	 output3 = null ;

        try {

            FileInputStream  inputA  = new FileInputStream(source);
            FileReader 		 inputB  = new FileReader(source);
			FileOutputStream outputA = new FileOutputStream(sink1);
            FileWriter 		 outputB = new FileWriter(sink2);
            FileOutputStream outputC = new FileOutputStream(sink3);

            input1 = new BufferedInputStream(inputA) ;			// pointing to "input1.txt"
            input2 = new BufferedReader(inputB) ;				// pointing to "input1.txt"
            output1 = new BufferedOutputStream(outputA) ;		// pointing to "output1.txt"
            output2 = new BufferedWriter(outputB) ;				// pointing to "output2.txt"
            output3	= new DataOutputStream(outputC);			// pointing to "output3.txt"

        } catch (FileNotFoundException filenotfound) {
            System.out.println("Could not open files.");
            System.exit(0);
        }

		int count1 = 0 ;
		int count2 = 0 ;

        // Reading and writing using a buffered BYTE-TO-BYTE stream
        int valueRead;
        while ( (valueRead = input1.read()) != -1 ) {
			count1++ ;
			output1.write(valueRead);
		}

		// Reading and writing using a buffered CHAR-TO-CHAR reader/writer
		// Remark: readLine() checks for 'null' instead of '-1' (see API specification)
		String line ;
		while ( (line = input2.readLine()) != null )	{
			count2++ ;
			output2.write(line, 0, line.length());
			output2.newLine();
        }

		// Writing portable data in binary format (to save disk space)
		int area = count1 * count2 ;
        output3.writeInt(area);

        // Closing the used stream objects (because they wrap the others)
        input1.close();
        input2.close();
        output1.close();
        output2.close();
        output3.close();

        return area ;
    }

	// Reading from the portable data file and returning the value read
	public int readData() throws IOException {

        DataInputStream input = null;

        try {

            FileInputStream inputFIS = new FileInputStream(sink3);		// pointing to "output3.txt"
            input = new DataInputStream(inputFIS);

        } catch (FileNotFoundException filenotfound) {
            System.out.println("Could not open the INPUT file.");
            System.exit(0);
        }

        // Read data written in binary form as Integer
        int valueRead = 0;
        while (true) {
            try 						{ 	valueRead = input.readInt();	}
            catch (EOFException eof) 	{ 	break;							}
        }

        input.close();
        return valueRead ;
	}

} // End of class