/**
 * Title: Expert 10 - Experimenting the JDBC paradigm: analyzing the database
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;

public class Expert10 {

	public static void main(String args[]) {

		 /* Specifying the DB location, to set this url you MUST verify following settings:
		 * Control Panel - Administration Tools - ODBC, then select the MDB data source and
		 * choose "Configuration". Here you will need to insert following data:
		 *
		 * 1. Data source name: specify "test" (instead of "Microsoft Access" or similar)
		 * 2. Database: choose "Browse" and specify the MDB file you want to connect to.
		 */
		String url   = "jdbc:odbc:test";

		// WE TRY TO GET THE JDBC DRIVER FOR MS ACCESS
		try {
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}


		// WE TRY TO CONNECT TO THE DB AND PERFORM THE QUERY
		try {

			// Here user & password are not required (""), hence we could use getConnection(url) ;
			Connection con = DriverManager.getConnection(url, "", "");
			checkForWarning(con.getWarnings());

			DatabaseMetaData database = con.getMetaData() ;

			System.out.println();
			System.out.println("Connected to         " + database.getURL());
			System.out.println("Driver               " + database.getDriverName());
			System.out.println("Version              " + database.getDriverVersion() + "\n");
			System.out.println("String functions:    " + database.getStringFunctions());
			System.out.println("Stored procedures:   " + database.supportsStoredProcedures());
			System.out.println("Outer Joins:         " + database.supportsOuterJoins());
			System.out.println("Full Outer Joins:    " + database.supportsFullOuterJoins());
			System.out.println("Limited Outer Joins: " + database.supportsLimitedOuterJoins() + "\n");

			// IMPORTANT: we need to close the connection object, in order to allow the GC to
			// delete it. In fact the GC can NOT know when an EXTERNAL resource is not used anymore !!!
			con.close();

		} catch (SQLException ex) {

			System.out.println ("\n*** SQLException caught ***\n");

			while (ex != null) {
				System.out.println("SQLState: " + ex.getSQLState ());
				System.out.println("Message:  " + ex.getMessage ());
				System.out.println("Vendor:   " + ex.getErrorCode () + "\n");
				ex = ex.getNextException ();
			}

		} catch (java.lang.Exception ex) {	ex.printStackTrace ();		}
	}

	// If a SQLWarning object was given, we display the warning messages.
	// Note that there could be multiple warnings chained together.
	private static boolean checkForWarning (SQLWarning warn) throws SQLException  {

		boolean rc = false;

		if (warn != null) {
			System.out.println ("\n *** Warning ***\n");
			rc = true;
			while (warn != null) {
				System.out.println ("SQLState: " + warn.getSQLState ());
				System.out.println ("Message:  " + warn.getMessage ());
				System.out.println ("Vendor:   " + warn.getErrorCode ());
				System.out.println ("");
				warn = warn.getNextWarning ();
			}
		}

		return rc;
	}

} // End of class
