/**
 * Title: Expert 11 - Experimenting the JDBC paradigm: executing queries
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Moprheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;

public class Expert11 {

	// Main method
	public static void main(String args[]) {

		String url   = "jdbc:odbc:test" ;
		String query = "SELECT * from Dreamsigns where dreamsign_id < 10" ;

		try {
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver") ;

		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ") ;
			System.err.println(e.getMessage()) ;
		}

		try {
			Connection con = DriverManager.getConnection(url, "", "") ;
			Statement stmt = con.createStatement() ;
	   		ResultSet result = stmt.executeQuery(query) ;
			Expert11.display(result) ;

			// Releasing the resources in order to help the Gargage Collector
			stmt.close();
			con.close();

		} catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
		}

	}

	// Display data, fetching rows until the end of the result set.
	// This method is statis because it will be used by next exercises!
	public static void display(ResultSet result) throws SQLException {

		ResultSetMetaData data = result.getMetaData() ;
		int numCols = data.getColumnCount();

		// Note the the cursor, before opening the result set, points BEFORE the first row.
		// Hence, by calling next() the cursor moves to first row of this Result Set.
		boolean more = result.next ();

		// Reading and printing column names
		for (int i = 1; i <= numCols; i++) {
			if (i > 1) System.out.print(" - ");
			System.out.print(data.getColumnName(i)) ; 	// alternative: getColumnLabel(i)
		}
		System.out.println("");

		// Reading and printing data from the ResultSet object
		int count = 1 ;
		while (more) {

			System.out.print(count++   +   "> ");

			for (int j = 1; j <= numCols; j++) {
				if (j > 1) System.out.print(" - ");
				System.out.print(result.getString(j));
			}

			System.out.println("");
			more = result.next ();
		}
	}

} // End of class
