/**
 * Title: Expert 12 - Experimenting the JDBC paradigm: Batch Updates
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;

public class Expert12 {

	// Run this application with parameter args[0] = "c" to create the tables.
	public static void main(String args[]) {

		String url    = "jdbc:odbc:test" ;
		String table1 = "create table PEOPLE (NAME varchar(20), PERSON_ID int, AGE int)" ;
		String table2 = "create table TASK (DESCRIPTION varchar(30), TASK_ID int, PERSON int, DONE bit)" ;

		try {
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
		} catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		try {
			Connection con = DriverManager.getConnection(url);
			Statement stmt = con.createStatement();

			// Creates the tables only when running like "java Expert12 c"
	   		if ( (args.length > 0) && (args[0].equals("c")) ) {
	   			stmt.executeUpdate(table1);
	   			stmt.executeUpdate(table2);
			}

			DatabaseMetaData database = con.getMetaData() ;
    		if(database.supportsBatchUpdates()) {

				con.setAutoCommit(false);
				stmt.addBatch("INSERT INTO people VALUES ('Steve', 17, 33)");
				stmt.addBatch("INSERT INTO people VALUES ('Thomas', 18, 31)");
				stmt.addBatch("INSERT INTO task VALUES ('Compile this example', 1, 17, true)");
				stmt.addBatch("INSERT INTO task VALUES ('Run this example', 2, 18, false)");

				// Executing the batch list as a transaction. Note that only command returning a single
				// count number can be used in a batch list (a SELECT statement can not be used here)
				int [] updateCounts = stmt.executeBatch();

			} else System.err.print("Error: the driver does not support batch commands") ;

			// Following 2 lines are not necessary here, they will be useful for next exercises.
			System.out.println("Press Enter to COMMIT this batch transaction") ;
			int foo = System.in.read() ;

			con.commit();
			stmt.close();
			con.close();
		} catch(BatchUpdateException buex) {

			System.err.print("Batch Exception!\nOnly these commands have been executed: ");
			int[] counts = buex.getUpdateCounts();
			for (int i = 0; i < counts.length; i ++) System.err.print(counts[i] + " - ");
			if (counts.length == 0) System.err.print("none");
			System.err.println("");
		}
		catch(SQLException ex) {	System.err.println("SQLException: "  + ex.getMessage()); }
		catch(Exception ex)    {	System.err.println("I/O Exception: " + ex.getMessage()); }
	}

} // End of class
