/**
 * Title: Expert 14 - JDBC paradigm: deleting rows using ResultSet; using wasNull()
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;

public class Expert14 {

	int count, row ;

	public static void main(String args[]) {

		try {	Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");		}
		catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		Expert14 manager = new Expert14() ;
		manager.init() ;		// Here the connection is opened
		manager.delete() ;
		manager.checkNull() ;	// Here the connection is closed
	}

	// Initializes the connection and analyze the database's metadata
	public void init() {

		Connection con = null ;
		Statement stmt = null ;

		try {
			con = DriverManager.getConnection("jdbc:odbc:test","","");
			con.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
			stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);

			DatabaseMetaData database = con.getMetaData() ;
			System.out.println() ;
			System.out.println("Supports Positioned Delete? " + database.supportsPositionedDelete()) ;
			System.out.println("Supports Positioned Update? " + database.supportsPositionedUpdate()) ;
			System.out.println() ;
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
	}

	// Shows rows from PEOPLE and allow the user to delete one row
	public void delete() {

		Connection con = null ;
		Statement stmt = null ;
		String query = "SELECT * FROM people" ;

		try {
			con = DriverManager.getConnection("jdbc:odbc:test", "", "");
			stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rs = stmt.executeQuery(query);
			Expert11.display(rs) ;
			rs.last() ;
			count = rs.getRow() ;

			System.out.print("\nInsert row number to be deleted (insert 0 to exit): ") ;
			row = Expert13.getInput() ;

			if ( (row > 0) && (row <= count) ) {
				rs.absolute(row) ;
				rs.deleteRow() ;		// Positioned delete !
			}
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
	}

	// Retrieve data from TASK and look for NULL values
	public void checkNull() {

		Connection con = null ;
		Statement stmt = null ;
		String query = "SELECT * FROM task" ;

		try {
			System.out.println("\nLooking for NULL values in table TASK:") ;
			con = DriverManager.getConnection("jdbc:odbc:test","","");
			stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rs = stmt.executeQuery(query);

			int count = 0 ;
			boolean more = rs.next();
			while (more) {

				count++ ;

				// Let's assume we know the structure of this table (see Expert12). We must use wasNull()
				// to distinguish actual NULL values from "0" (int value) and "false" (boolean value).
				// Remark: wasNull() refers to the LAST value read from the ResultSet !!!
				if (rs.getString(1) == null)
					System.out.println("Row n." + count + ": Null DESCRIPTION value") ;
				if (rs.getInt(2) == 0)
					if (rs.wasNull()) System.out.println("Row n." + count + ": Null TASK_ID value") ;
				if (rs.getInt(3) == 0)
					if (rs.wasNull()) System.out.println("Row n." + count + ": Null PERSON value") ;
				if (!rs.getBoolean(4))
					if (rs.wasNull()) System.out.println("Row n." + count + ": Null DONE value") ;

				more = rs.next ();
			}

			con.commit() ;
			stmt.close();
			con.close();
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
	}

} // End of class
