/**
 * Title: Expert 15 - JDBC paradigm: retrieving very large row values
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.sql.*;
import java.io.*;

public class Expert15 {

	int count, row ;

	public static void main(String args[]) {

		try {	Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");		}
		catch(java.lang.ClassNotFoundException e) {
			System.err.print("ClassNotFoundException: ");
			System.err.println(e.getMessage());
		}

		Expert15 manager = new Expert15() ;
		manager.readData() ;
	}

	// Reading data using getBinaryStream() and getAsciiStream().
	public void readData() {

		Connection con = null ;
		Statement stmt = null ;
		InputStream fin ;
		String query = "SELECT * FROM Journal where dream_id = 9" ;
		byte [] buffer = new byte[128]; 	// Retrieving results in 128 bytes chunks

		try {
			con = DriverManager.getConnection("jdbc:odbc:test", "", "");
			stmt = con.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rs = stmt.executeQuery(query);
			System.out.println("\nBINARYSTREAM:");

			// Reading using BINARYSTREAM
			while (rs.next()) {

				fin = rs.getBinaryStream(3);			// Reading 3rd column

				// Using and endless for cycle
				for (;;) {
					int size = fin.read(buffer);
					if (size == -1) break; 				// End of stream
					// Send the newly filled buffer to the System.out output stream
					System.out.write(buffer, 0, size);
					System.out.print("<END>");
				}
			}

			rs.beforeFirst() ;
			System.out.println("\n\n************************\n");
			System.out.println("ASCIISTREAM:");

			// Reading using ASCIISTREAM
			while (rs.next()) {

				fin = rs.getAsciiStream(3);				// Reading 3rd column

				for (;;) {
					int size = fin.read(buffer);
					if (size == -1) break; 				// End of stream
					// Send the newly-filled buffer to the System.out output stream
					System.out.write(buffer, 0, size);
					System.out.print("<END>");
				}
			}

			stmt.close();
			con.close();
		}
		catch(SQLException ex) {
			System.err.println("SQLException: " + ex.getMessage());
			ex.printStackTrace() ;
		}
		catch(Exception ex)    {	System.err.println("Exception reading buffer: " + ex.getMessage()); }
	}

} // End of class