/**
 * Title: Expert 16 - An appletcation example.
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public class Expert16 extends Applet implements ActionListener	{

// ColorControl is defined as an inner class (see below).
ColorControl[] control = new ColorControl[3];

	// Constructor
    public Expert16() {

        control[0] = new ColorControl("Red");
        control[1] = new ColorControl("Green");
        control[2] = new ColorControl("Blue");

        // We'll use 3 rows for the ColorControl's, and the 3 rows for the "color preview".
        setLayout( new GridLayout(6,1) );
        for (int i = 0; i < 3; i++) add(control[i]);
    }

	// Called when the user press enter in the TextField's (see the ColorControl class).
    public void actionPerformed(ActionEvent theEvent) {

        Color c = new Color(control[0].getValue(), control[1].getValue(), control[2].getValue());
        setBackground(c);
        repaint();
    }

	// Main method
    public static void main(String arg[]) {

        Expert16 theApplet = new Expert16( );
        AppletFrame af = new AppletFrame(theApplet, "The Expert16 Applet");

        af.add(theApplet,"Center");
        af.setSize(200, 300);
        af.setVisible(true);
        theApplet.init();
        theApplet.start();
    }

} // END OF EXPERT16 APPLET


/* 	Inner applet which contains the control (the label and the text field).
	It is created "outside" the Expert16 only for training reason, in order to explain
	how to forward Events to the "parent container" (see getParent() below).		*/
class ColorControl extends Applet implements ActionListener	{

	private Label name;
	private int value = 255;
	private TextField input = new TextField("" + value);

	// Constructor.
    public ColorControl(String s) {

        name = new Label(s) ;
	    setLayout(new GridLayout(1,2)) ;
        add(name) ;
        add(input) ;
        input.addActionListener(this) ;
    }

	// Called when the user press "Enter" on the TextField.
    public void actionPerformed(ActionEvent theEvent) {

        value = Integer.parseInt(input.getText()) ;
        if (value > 255) value = 255 ;
        input.setText( "" + value ) ;

		// By this mechanism the action is forwarded to the parent.
		// Try to comment this line and re-compile: the color will not change !!!
        ( (ActionListener) getParent() ).actionPerformed(theEvent) ;
    }

    public int getValue() { return value ; }

} // END OF COLORCONTROL APPLET


// 	Inner application that takes care to manage the Expert16 applet.
class AppletFrame extends Frame implements WindowListener {

	Applet theApplet;

	// Constructor.
    public AppletFrame(Applet anyApplet, String title) {

        super(title);
        theApplet = anyApplet;
        addWindowListener(this);
    }

	/* Try to comment these lines: this code will not react if you close the window! 	*/
    public void windowClosed     (WindowEvent theEvent) {  }
    public void windowClosing    (WindowEvent theEvent) { quitApplication( ); }
    public void windowDeiconified(WindowEvent theEvent) { }
    public void windowIconified  (WindowEvent theEvent) { }
    public void windowOpened     (WindowEvent theEvent) { }
    public void windowActivated  (WindowEvent theEvent) { }
    public void windowDeactivated(WindowEvent theEvent) { }

    public void quitApplication( )    {

        setVisible(false);			// Hides the "window's frame", but NOT the frame itself.
        theApplet.setVisible(false);
        theApplet.stop();
        theApplet.destroy();
        dispose();					// destroy the Frame (similar to the destroy() method for applet).
        System.exit(0);
    }

} // END OF THE FRAME CLASS
