/**
 * Title: Expert 18 - A basic AWT men
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.awt.*;
import java.awt.event.*;

public class Expert18 extends Frame implements ActionListener, WindowListener, ItemListener {

    MenuBar   theBar	= new MenuBar();
    Menu      fileMenu	= new Menu("File");
    Menu      editMenu	= new Menu("Edit");
    Menu      helpMenu	= new Menu("Help");
    MenuItem  newItem, openItem, closeItem, exitItem ;
    MenuItem  copyItem, cutItem, pasteItem, helpItem;

    Menu channelMenu ;						// Used in the second part of the exercise
    CheckboxMenuItem alpha, beta, gamma ;	// Used in the second part of the exercise

	// Main method
    public static void main(String args[]) { 	Expert18 theApp = new Expert18();    }

	// Constructor.
    public Expert18() {

        super("Basic AWT Menu");	// Calling the Frame's super constructor

        // Creating all Menu items
		newItem 	= new MenuItem("New") ;
		openItem 	= new MenuItem("Open") ;
		closeItem 	= new MenuItem("Close") ;
		exitItem 	= new MenuItem("Exit") ;
		copyItem 	= new MenuItem("Copy") ;
		cutItem 	= new MenuItem("Cut") ;
		pasteItem 	= new MenuItem("Paste") ;
		helpItem	= new MenuItem("Help") ;

		// Adding the main menu entry
        setMenuBar(theBar);
        theBar.add(fileMenu);
        theBar.add(editMenu);
        theBar.setHelpMenu(helpMenu);

		// Adding items to the File menu
        fileMenu.add(newItem);
        fileMenu.add(openItem);
        fileMenu.add(closeItem);
        fileMenu.addSeparator();
        fileMenu.add(exitItem);

		// Adding items to the Edit menu
        editMenu.add(copyItem);
        editMenu.add(cutItem);
        editMenu.add(pasteItem);
		copyItem.setEnabled(false) ;
		cutItem.setEnabled(false) ;
		pasteItem.setEnabled(false) ;

        // Adding just one item to the Help menu
		helpMenu.add(helpItem);

		// Defining listeners
        addWindowListener(this);
		newItem.addActionListener(this);
		openItem.addActionListener(this);
		closeItem.addActionListener(this);
        exitItem.addActionListener(this);
        helpItem.addActionListener(this);

		// Defining layout
        setSize(500, 300);
        setLocation(200, 150);
        setVisible(true);
    }

	// Handling the various menu actions
    public void actionPerformed(ActionEvent event) {

       if ( event.getSource() == exitItem ) quitApplication() ;
       if ( event.getSource() == helpItem ) System.out.println("Sorry, no help") ;
       	   else otherEntry(event) ;		// see the second part
    }

	// WindowListener methods (mandatory)
    public void windowClosed     (WindowEvent theEvent) {  }
    public void windowClosing    (WindowEvent theEvent) { quitApplication( ); }
    public void windowDeiconified(WindowEvent theEvent) { }
    public void windowIconified  (WindowEvent theEvent) { }
    public void windowOpened     (WindowEvent theEvent) { }
    public void windowActivated  (WindowEvent theEvent) { }
    public void windowDeactivated(WindowEvent theEvent) { }

	// Quit the application
    public void quitApplication() {

        setVisible(false);
        dispose();			// Free all system's native resources
        System.exit(0);
    }


// **********************************	SECOND PART 	*******************************************


	// Handling other menu actions: Open and Close
    public void otherEntry(ActionEvent event) {

		if ( event.getSource() == openItem ) changeMode(true) ;
		if ( event.getSource() == closeItem ) changeMode(false) ;
	}

	// Enabling or disabling the Channel menu
	public void changeMode(boolean mode) {

		// Changing state of the "Edit" menu
		copyItem.setEnabled(mode) ;
		cutItem.setEnabled(mode) ;
		pasteItem.setEnabled(mode) ;

		// Creating the "Channel" menu on the fly (only 1st time)
		if (channelMenu == null) {

			channelMenu = new Menu("Channels") ;
			alpha		= new CheckboxMenuItem("Alpha channel") ;
			beta		= new CheckboxMenuItem("Beta channel") ;
			gamma		= new CheckboxMenuItem("Gamma channel") ;

			channelMenu.add(alpha) ;
			channelMenu.add(beta) ;
			channelMenu.add(gamma) ;
			editMenu.add(channelMenu) ;
			alpha.addItemListener(this) ;
			beta.addItemListener(this) ;
			gamma.addItemListener(this) ;
		}

		// Adding or removing the "channelMenu"
		if (channelMenu != null) {
			if (mode) editMenu.add(channelMenu) ;
		  	  else editMenu.remove(channelMenu) ;
	    }
	}

	// Called when a CheckBox item is changed
	public void itemStateChanged(ItemEvent event) {

		if ( event.getSource() == alpha ) 	System.out.println("Entered alpha channel") ;
		if ( event.getSource() == beta ) 	System.out.println("Entered beta channel") ;
		if ( event.getSource() == gamma ) 	System.out.println("Entered gamma channel") ;

		// setState(true) is called by Java when the user clicks on the item, so following
		// code is needed only to UNSELECT the items that were not choosen!
		// For this reason, without following code multiple selection is possible !
		if (alpha != null)	alpha.setState(event.getSource() == alpha) ;
		if (beta != null)	beta.setState(event.getSource() == beta) ;
		if (gamma != null)	gamma.setState(event.getSource() == gamma) ;
	}

} // END OF CLASS
