/**
 * Title: Expert 19 - A simple File Dialog window.
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:      Morpheus
 * @author Stefano Adriani
 * @version 1.0
 */

package adriani.tutorial.expert ;

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class Expert19 extends Frame implements FilenameFilter, ActionListener, WindowListener {


	// Note: the FileDialog constructor wants a reference to the parent frame (this)
    Label    	theLabel		= new Label();								// Used to show the selection
    TextArea 	theDisplay		= new TextArea();							// Used to show the file's content
    FileDialog	theFileDialog	= new FileDialog(this, "Open File");
    Button   	theButton		= new Button("Click here to open the File Dialog");

    char 		fp				= File.separatorChar ;
    String		theFolder		= "." ;
    String		thePath			= "C:" + fp + "Work" + fp + "Java" ;		// Default path when opened


	// The main method
    public static void main(String args[])    {

		Expert19 theApp = new Expert19();
    }


	// Costructor.
    public Expert19()    {

		// Note: Expert19 implements the FilenameFilter interface!
        theFileDialog.setFilenameFilter(this);			// In this way the FileDialog will call this.accept()
        setTitle("Example of FileDialog");
        setLayout(new BorderLayout());
        setSize(600,400);

        add(theButton,"South");
        add(theDisplay,"Center");
        add(theLabel,"North");

        theButton.addActionListener(this);				// Opens the FileDialog window
        addWindowListener(this);
        setVisible(true);
    }


	// Called when the FileDialog is opened.
    public void actionPerformed(ActionEvent theEvent)    {

		theLabel.setText("Directory: ");
		theFileDialog.setDirectory(thePath);
		theFileDialog.setDirectory(theFolder);
        //theFileDialog.setFile("Expert19.java");		// Provides a default selection

		// Following "System.out"'s show where the dialog "stop" waiting for user's input:
        System.out.println("Making visibile the File dialog ...") ;
        theFileDialog.setVisible(true);
        System.out.println("Making NOT visibile the File dialog ...") ;
        theFileDialog.setVisible(false);

		// Retrieving the selection from the File Dialog
        int mode = theFileDialog.getMode() ;
        String file = theFileDialog.getFile() ;
        String folder = theFileDialog.getDirectory() ;
        theLabel.setText("Directory: " + folder) ;

		// Accepting the file with the right extension (see below)
        if ( (file != null) && (accept(new File(folder),file)) ) {
			displayFile(folder, file);
        	System.out.println("Opening file in mode: " + mode) ;		// Just for training reasons
		}
        setVisible(true);
    }


	// Shows the file content on "theDisplay" TextArea
    public void displayFile(String dirName, String fileName)    {

        FileInputStream inFile = null;

        try {
            inFile = new FileInputStream(dirName + fileName) ;
            byte[] buffer = new byte[inFile.available()] ;
            inFile.read(buffer) ;
            theDisplay.append("" + new String(buffer, 0, buffer.length)) ;

        } catch (IOException e) { System.exit(1) ; }
    }


	// We MUST implemet this method because it is defined in the FilenameFilter interface
    public boolean accept(File dir, String name)    {

        System.out.println("Testing accept() on: " + name);
        if (name.endsWith(".txt")) 	return true;
        if (name.endsWith(".bat")) 	return true;
        if (name.endsWith(".java")) return true;

        return false;
    }


	// We MUST implemet these methods because it is defined in the WindowListener interface
    public void windowClosed     (WindowEvent theEvent)	{  }
    public void windowClosing    (WindowEvent theEvent)	{ quitApplication( ); }
    public void windowDeiconified(WindowEvent theEvent)	{ }
    public void windowIconified  (WindowEvent theEvent)	{ }
    public void windowOpened     (WindowEvent theEvent)	{ }
    public void windowActivated  (WindowEvent theEvent)	{ }
    public void windowDeactivated(WindowEvent theEvent)	{ }

    public void quitApplication( )    {

        setVisible(false );
        dispose( );
        System.exit(0);
    }

} // End of class

